"use strict";
require("reflect-metadata");
var path = require("path");
var configFile = path.join(process.cwd(), "typeunit.config");
exports.config = require(configFile);
var FACT_METADATA_KEY = "type-unit:facts";
var THEORY_METADATA_KEY = "type-unit:theories";
exports.fact = function (description) {
    return function (target, propertyKey, descriptor) {
        var facts = Reflect.getMetadata(FACT_METADATA_KEY, target);
        if (!facts) {
            facts = [];
            Reflect.defineMetadata(FACT_METADATA_KEY, facts, target);
        }
        var fact = {
            fn: target[propertyKey],
            description: description
        };
        facts.push(fact);
    };
};
exports.theory = function (datasets, description) {
    return function (target, propertyKey, descriptor) {
        var theories = Reflect.getMetadata(THEORY_METADATA_KEY, target);
        if (!theories) {
            theories = [];
            Reflect.defineMetadata(THEORY_METADATA_KEY, theories, target);
        }
        var theory = {
            fn: target[propertyKey],
            datasets: datasets,
            description: description
        };
        theories.push(theory);
    };
};
exports.suite = function (description) {
    return function (target) {
        var staticFacts = Reflect.getMetadata(FACT_METADATA_KEY, target);
        var instanceFacts = Reflect.getMetadata(FACT_METADATA_KEY, target.prototype);
        var staticTheories = Reflect.getMetadata(THEORY_METADATA_KEY, target);
        var instanceTheories = Reflect.getMetadata(THEORY_METADATA_KEY, target.prototype);
        var testsToInvoke = [];
        if (staticFacts) {
            var _loop_1 = function(i) {
                var suiteInstance = new target();
                var staticFact = staticFacts[i];
                testsToInvoke.push(function () { return invokeTest(staticFact.fn, global, staticFact.description); });
            };
            for (var i = 0; i < staticFacts.length; i++) {
                _loop_1(i);
            }
        }
        if (instanceFacts) {
            var _loop_2 = function(i) {
                var suiteInstance = new target();
                var instanceFact = instanceFacts[i];
                testsToInvoke.push(function () { return invokeTest(instanceFact.fn, suiteInstance, instanceFact.description); });
            };
            for (var i = 0; i < instanceFacts.length; i++) {
                _loop_2(i);
            }
        }
        if (staticTheories) {
            var _loop_3 = function(i) {
                var staticTheory = staticTheories[i];
                var _loop_4 = function(j) {
                    var dataset = staticTheory.datasets[j];
                    testsToInvoke.push(function () { return invokeTest(staticTheory.fn, global, staticTheory.description, dataset); });
                };
                for (var j = 0; j < staticTheory.datasets.length; j++) {
                    _loop_4(j);
                }
            };
            for (var i = 0; i < staticTheories.length; i++) {
                _loop_3(i);
            }
        }
        if (instanceTheories) {
            var _loop_5 = function(i) {
                var suiteInstance = new target();
                var instanceTheory = instanceTheories[i];
                var _loop_6 = function(j) {
                    var dataset = instanceTheory.datasets[j];
                    testsToInvoke.push(function () { return invokeTest(instanceTheory.fn, suiteInstance, instanceTheory.description, dataset); });
                };
                for (var j = 0; j < instanceTheory.datasets.length; j++) {
                    _loop_6(j);
                }
            };
            for (var i = 0; i < instanceTheories.length; i++) {
                _loop_5(i);
            }
        }
        runSuite(target, description, testsToInvoke);
    };
};
var invokeTest = function (test, thisContext, description, params) {
    var fn = function () {
        test.apply(thisContext, params);
    };
    if (exports.config.plugin && exports.config.plugin.test) {
        exports.config.plugin.test(fn, params, description);
    }
    else {
        fn();
    }
};
var runSuite = function (suiteInstance, description, testsToInvoke) {
    var isPluginPresent = exports.config.plugin && exports.config.plugin.suite;
    if (isPluginPresent) {
        exports.config.plugin.suite(suiteInstance, testsToInvoke, description);
    }
    else {
        for (var i = 0; i < testsToInvoke.length; i++) {
            var test_1 = testsToInvoke[i];
            test_1();
        }
    }
};
//# sourceMappingURL=index.js.map